@@ -12,6 +12,7 @@ from .queue import Queue
 from .scheduler import CronTask
 from .serializer import deserialize, serialize
 from .task import Task
+from .utils import safe_eval
 
 __all__ = [
     "WakaQ",
@@ -30,11 +31,16 @@ class WakaQ:
     wait_timeout = None
     max_mem_percent = None
     max_tasks_per_worker = None
-    log_file = None
-    log_level = None
-    after_worker_startup = None
+    worker_log_file = None
+    scheduler_log_file = None
+    worker_log_level = None
+    scheduler_log_level = None
+
+    after_worker_started_callback = None
+    before_task_started_callback = None
+    after_task_finished_callback = None
+    wrap_tasks_function = None
 
-    eta_task_key = "wakaq-eta"
     broadcast_key = "wakaq-broadcast"
     log_format = "[%(asctime)s] %(levelname)s: %(message)s"
     task_log_format = "[%(asctime)s] %(levelname)s in %(task)s: %(message)s"
@@ -51,9 +57,10 @@ class WakaQ:
         hard_timeout=None,
         max_mem_percent=None,
         max_tasks_per_worker=None,
-        after_worker_startup=None,
-        log_file=None,
-        log_level=None,
+        worker_log_file=None,
+        scheduler_log_file=None,
+        worker_log_level=None,
+        scheduler_log_level=None,
         socket_timeout=15,
         socket_connect_timeout=15,
         health_check_interval=30,
@@ -66,7 +73,7 @@ class WakaQ:
         self.queues_by_name = dict([(x.name, x) for x in self.queues])
         self.broker_keys = [x.broker_key for x in self.queues]
         self.schedules = [CronTask.create(x) for x in schedules]
-        self.concurrency = abs(int(concurrency)) or multiprocessing.cpu_count()
+        self.concurrency = self._format_concurrency(concurrency)
         self.exclude_queues = self._validate_queue_names(exclude_queues)
         self.soft_timeout = soft_timeout.total_seconds() if isinstance(soft_timeout, timedelta) else soft_timeout
         self.hard_timeout = hard_timeout.total_seconds() if isinstance(hard_timeout, timedelta) else hard_timeout
@@ -93,12 +100,10 @@ class WakaQ:
             self.max_mem_percent = None
 
         self.max_tasks_per_worker = abs(int(max_tasks_per_worker)) if max_tasks_per_worker else None
-        self.log_file = log_file if isinstance(log_file, str) else None
-        self.log_level = log_level if isinstance(log_level, int) else logging.INFO
-
-        if after_worker_startup and not callable(after_worker_startup):
-            raise Exception("after_worker_startup must be a function")
-        self.after_worker_startup = after_worker_startup
+        self.worker_log_file = worker_log_file if isinstance(worker_log_file, str) else None
+        self.scheduler_log_file = scheduler_log_file if isinstance(scheduler_log_file, str) else None
+        self.worker_log_level = worker_log_level if isinstance(worker_log_level, int) else logging.INFO
+        self.scheduler_log_level = scheduler_log_level if isinstance(scheduler_log_level, int) else logging.DEBUG
 
         self.tasks = {}
         self.broker = redis.Redis(
@@ -121,6 +126,22 @@ class WakaQ:
 
         return wrap(fn) if fn else wrap
 
+    def after_worker_started(self, callback):
+        self.after_worker_started_callback = callback
+        return callback
+
+    def before_task_started(self, callback):
+        self.before_task_started_callback = callback
+        return callback
+
+    def after_task_finished(self, callback):
+        self.after_task_finished_callback = callback
+        return callback
+
+    def wrap_tasks_with(self, callback):
+        self.wrap_tasks_function = callback
+        return callback
+
     def _validate_queue_names(self, queue_names: list) -> list:
         try:
             queue_names = [x for x in queue_names]
@@ -153,18 +174,19 @@ class WakaQ:
         )
         self.broker.rpush(queue.broker_key, payload)
 
-    def _enqueue_with_eta(self, task_name: str, queue: str, args: list, kwargs: dict, eta: timedelta):
+    def _enqueue_with_eta(self, task_name: str, queue: str, args: list, kwargs: dict, eta):
         queue = self._queue_or_default(queue)
         payload = serialize(
             {
                 "name": task_name,
                 "args": args,
                 "kwargs": kwargs,
-                "queue": queue.name,
             }
         )
-        timestamp = calendar.timegm((datetime.utcnow() + eta).utctimetuple())
-        self.broker.zadd(self.eta_task_key, {payload: timestamp}, nx=True)
+        if isinstance(eta, timedelta):
+            eta = datetime.utcnow() + eta
+        timestamp = calendar.timegm(eta.utctimetuple())
+        self.broker.zadd(queue.broker_eta_key, {payload: timestamp}, nx=True)
 
     def _broadcast(self, task_name: str, queue: str, args: list, kwargs: dict):
         queue = self._queue_or_default(queue)
@@ -193,3 +215,11 @@ class WakaQ:
         if queue.priority < 0:
             queue.priority = lowest_priority + 1
         return queue
+
+    def _format_concurrency(self, concurrency):
+        if not concurrency:
+            return 0
+        try:
+            return safe_eval(str(concurrency), {"cores": multiprocessing.cpu_count()})
+        except Exception as e:
+            raise Exception(f"Error parsing concurrency: {e}")
