@@ -10,7 +10,6 @@ import sys
 import time
 import traceback
 
-import daemon
 import psutil
 
 from .exceptions import SoftTimeout
@@ -72,16 +71,10 @@ class Worker:
     def __init__(self, wakaq=None):
         self.wakaq = wakaq
 
-    def start(self, foreground=False):
+    def start(self):
         setup_logging(self.wakaq)
         log.info(f"starting {self.wakaq.concurrency} workers")
-
-        if foreground:
-            self._run()
-            return
-
-        with daemon.DaemonContext():
-            self._run()
+        self._run()
 
     def _stop(self):
         self._stop_processing = True
@@ -120,10 +113,13 @@ class Worker:
         signal.signal(signal.SIGCHLD, self._on_child_exited)
         signal.signal(signal.SIGINT, self._on_exit_parent)
         signal.signal(signal.SIGTERM, self._on_exit_parent)
+        signal.signal(signal.SIGQUIT, self._on_exit_parent)
 
         self._pubsub = self.wakaq.broker.pubsub()
         self._pubsub.subscribe(self.wakaq.broadcast_key)
 
+        log.info("finished forking all workers")
+
         while not self._stop_processing:
             self._read_child_logs()
             self._check_max_mem_percent()
@@ -172,23 +168,28 @@ class Worker:
             self._remove_all_children()
 
             log.debug("started worker process")
-            if self.wakaq.after_worker_startup:
-                self.wakaq.after_worker_startup()
+
+            if self.wakaq.after_worker_started_callback:
+                self.wakaq.after_worker_started_callback()
 
             self._num_tasks_processed = 0
             while not self._stop_processing:
                 self._execute_next_task_from_queue()
+                if self.wakaq.max_tasks_per_worker and self._num_tasks_processed >= self.wakaq.max_tasks_per_worker:
+                    log.debug(f"restarting worker after {self._num_tasks_processed} tasks")
+                    self._stop_processing = True
+                sys.stdout.flush()
 
         except:
             log.error(traceback.format_exc())
-            return
 
         finally:
-            for handler in log.handlers:
-                handler.flush()
+            sys.stdout.flush()
+            sys.stderr.flush()
             sys.stdout = sys.__stdout__
             sys.stderr = sys.__stderr__
-            fh.close()
+            os.close(self._stdout)
+            os.close(self._pingout)
 
     def _add_child(self, pid, pingin, stdin):
         self.children.append(Child(pid, pingin, stdin))
@@ -213,7 +214,6 @@ class Worker:
         self._stop_processing = True
 
     def _on_soft_timeout_child(self, signum, frame):
-        self._stop_processing = True
         raise SoftTimeout("SoftTimeout")
 
     def _on_child_exited(self, signum, frame):
@@ -229,14 +229,14 @@ class Worker:
 
     def _enqueue_ready_eta_tasks(self):
         script = self.wakaq.broker.register_script(ZRANGEPOP)
-        results = script(keys=[self.wakaq.eta_task_key], args=[int(round(time.time()))])
-        for payload in results:
-            payload = deserialize(payload)
-            task_name = payload.pop("name")
-            queue = payload.pop("queue")
-            args = payload.pop("args")
-            kwargs = payload.pop("kwargs")
-            self.wakaq._enqueue_at_front(task_name, queue, args, kwargs)
+        for queue in self.wakaq.queues:
+            results = script(keys=[queue.broker_eta_key], args=[int(round(time.time()))])
+            for payload in results:
+                payload = deserialize(payload)
+                task_name = payload.pop("name")
+                args = payload.pop("args")
+                kwargs = payload.pop("kwargs")
+                self.wakaq._enqueue_at_front(task_name, queue.name, args, kwargs)
 
     def _execute_next_task_from_queue(self):
         payload = self.wakaq._blocking_dequeue()
@@ -244,12 +244,20 @@ class Worker:
             task = self.wakaq.tasks[payload["name"]]
             current_task.set(task)
             log.debug(f"running with payload {payload}")
-            task.fn(*payload["args"], **payload["kwargs"])
-            current_task.set(None)
-            self._num_tasks_processed += 1
-            if self.wakaq.max_tasks_per_worker and self._num_tasks_processed >= self.wakaq.max_tasks_per_worker:
-                log.debug(f"restarting worker after {self._num_tasks_processed} tasks")
-                self._stop_processing = True
+            if self.wakaq.before_task_started_callback:
+                self.wakaq.before_task_started_callback()
+            try:
+                if self.wakaq.wrap_tasks_function:
+                    self.wakaq.wrap_tasks_function(task.fn)(*payload["args"], **payload["kwargs"])
+                else:
+                    task.fn(*payload["args"], **payload["kwargs"])
+            except:
+                log.error(traceback.format_exc())
+            finally:
+                current_task.set(None)
+                self._num_tasks_processed += 1
+                if self.wakaq.after_task_finished_callback:
+                    self.wakaq.after_task_finished_callback()
         os.write(self._pingout, b".")
 
     def _read_child_logs(self):
